
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <time.h>

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>

#include <arpa/inet.h>
#include <netinet/in.h> 


#include <string.h>
#include <netdb.h>

#include "udpshout.h"


/***********************************************
 * Static functions
 ***********************************************/
static int socket_open(char *laddr, int port) {
	struct sockaddr_in	lsin;
	int			sock;
	int			reuse=1;
	
	memset(&lsin, 0, sizeof(struct sockaddr_in));

	sock=socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP);

	if (sock<0)
		return sock;
		
	if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR,
                   (char *)&reuse, sizeof(reuse)) < 0) {	
		close(sock);
		return -1;
	}

	lsin.sin_family=AF_INET;
	lsin.sin_addr.s_addr=INADDR_ANY;
	if (laddr)
		inet_aton(laddr, &lsin.sin_addr);
	lsin.sin_port=htons(port);

	if (bind(sock, (struct sockaddr *) &lsin,
			sizeof(struct sockaddr_in)) != 0) {
		close(sock);
		return -1;
	}

	return sock;
}


/* Join the socket on a multicats group e.g. tell the kernel 
 * to send out IGMP join messages ...
 *
 * Returns 0 on success and != 0 in failure
 *
 */
static int socket_join_multicast(int sock, char *addr) {
	struct ip_mreq		mreq;

	memset(&mreq, 0, sizeof(struct ip_mreq));

	/* Its not an ip address ? */
	if (!inet_aton(addr, &mreq.imr_multiaddr))
		return -1;

	if (!IN_MULTICAST(ntohl(mreq.imr_multiaddr.s_addr)))
		return 0;

	mreq.imr_interface.s_addr=INADDR_ANY;

	return setsockopt(sock, IPPROTO_IP, IP_ADD_MEMBERSHIP, &mreq, sizeof(mreq));
}


/* ***************************************************
 * Shared functions
 * ***************************************************/
int input_init_udp(udpshout_channel_t *chan) {

	chan->udpsockfd=socket_open(chan->udp_ip, chan->udp_port);
	if(chan->udpsockfd < 0)
		goto errout1;
	
	
	/* Join Multicast group if its a multicast source */
	if (socket_join_multicast(chan->udpsockfd, chan->udp_ip) < 0)
		goto errout2;
	return 1;
	
	errout2:
		socket_close(chan->udpsockfd);
	errout1:
		return 0;
}

void socket_close(int sock) {
	close(sock);
}
